/*
 * A dictionnary
 *---
 * to my angel
 * public domain
 * march 2002
 */

#include "strings.h"

#include <string.h>
#include <stdio.h>

#include "mem.h"
#include "hash.h"
#include "verbose.h"

/* return 1 if ==, 0 if not */
static int string_compare(strings *s, char *new, user_data *in_hash)
{
  char *old = (char *)s->data + s->user_to_sid(s->user_to_sid_pointer, in_hash);

  return strcmp(new, old) == 0;
}

static unsigned long string_hash(strings *s, char *n)
{
  return hash_simple(&s->ht, n, strlen(n));
}

static char *string_get_key(strings *s, user_data *n)
{
  return (char *)s->data + s->user_to_sid(s->user_to_sid_pointer, n);
}

/* return -1 on error, 0 if all ok */
int init_strings(strings *s, string_user_to_sid_function f, user_context *cont, unsigned long user_data_length)
{
  memset(s, 0, sizeof(strings));

  init_hash(&s->ht, f == STRING_DEFAULT_USER_TO_SID ? sizeof(sid) : user_data_length, HASH_INIT_SIZE_SMALL,
	    (hash_compare_function)string_compare, s,
	    (hash_hash_function)string_hash, s,
	    (hash_get_key_function)string_get_key, s);
  if (s->ht.flag != HASH_NO_ERROR)
    return -1;

  s->user_to_sid = f;
  s->user_to_sid_pointer = f == STRING_DEFAULT_USER_TO_SID ? (void *)s : cont;

  return 0;
}

void free_strings(strings *s)
{
  print("hash", stdout, "free_strings : ");

  free_hash(&s->ht);

  if (s->maxsize)
    freemem(s->data);

  memset(s, 0, sizeof(strings));
}

/* return -1 if error, -2 if string already here, 0 if all ok, hid and sid set in both cases */
int string_new(strings *s, char *n, hid *ss, sid *sd)
{
  unsigned long nlen = strlen(n)+1;
  hid hid;

  hid = hash_get_new_slot(&s->ht, n);
  if (ss)
    *ss = hid;

  if (s->ht.flag == HASH_ALREADY_PRESENT) {
    if (sd)
      *sd = s->user_to_sid(s->user_to_sid_pointer, hash_get_slot_data(&s->ht, hid));
    return -2;
  }

  if (s->ht.flag != HASH_NO_ERROR) {
    fprintf(stderr, "strings.c:string_new(line %d) fatal error, unhandled hash error code %d\n", __LINE__, s->ht.flag);
    abort();
  }

  if (sd)
    *sd = s->size;

  if (s->size + nlen > s->maxsize) {
    s->maxsize = (s->maxsize+nlen+1024)&~1023;
    s->data = regetmem(s->data, s->maxsize * sizeof(char));
  }

  memcpy((char *)s->data + s->size, n, nlen * sizeof(char));
  s->size += nlen;

  return 0;
}

void string_feed_slot(strings *s, hid h, void *data)
{
  hash_feed_slot(&s->ht, h, data);
}

void *string_get_slot(strings *s, hid h)
{
  return hash_get_slot_data(&s->ht, h);
}

/* return -1 if string not here, 0 if yes and set its sid and user_data */
int string_search(strings *s, char *n, sid *ss, user_data **ud)
{
  hid hid;
  user_data *d;

  hid = hash_get_slot(&s->ht, n);

  if (s->ht.flag == HASH_NOT_PRESENT)
    return -1;

  if (s->ht.flag != HASH_NO_ERROR) {
    fprintf(stderr, "strings.c:string_search(line %d) fatal error, unhandled hash error code %d\n", __LINE__, s->ht.flag);
    abort();
  }

  d = hash_get_slot_data(&s->ht, hid);
  if (ud)
    *ud = d;
  if (ss)
    *ss = s->user_to_sid(s->user_to_sid_pointer, d);

  return 0;
}

char *sid_to_string(strings *s, sid n)
{
  return (char *)s->data + n;
}

sid string_user_to_sid(strings *unused, sid *s)
{
  return *s;
}
