/*
 * Some verbosity facilities
 * Only one verbosity per program, because of some globals.
 *---
 * to my angel
 * public domain
 * march 2002
 */

#include "verbose.h"

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#include "hash.h"
#include "mem.h"
#include "strings.h"

/* this file exist only if wanted */
#if (__STDC_VERSION__ && __STDC_VERSION__ >= 199901L && DEBUG) || !(__STDC_VERSION__ && __STDC_VERSION__ >= 199901L)

/* ugly globals */
static verbose *the_verbose;
static int inited = 0;
static int all = 0;

/* -1 if error, 0 if not */
int init_verbose(verbose *v)
{
  if (inited)
    return -1;

  if (init_strings(&v->s, STRING_DEFAULT_USER_TO_SID, 0, 0) == -1)
    return -1;

  inited = 1;
  the_verbose = v;

  return 0;
}

void free_verbose(verbose *v)
{
  if (!inited)
    return;

  free_strings(&v->s);

  all = 0;
  inited = 0;
}

/* returns -1 if error, 0 if all ok */
int verbose_register(verbose *v, char *c)
{
  int ret;
  hid h;
  sid s;

  if (!inited)
    return -1;

  if (!strcmp(c, "all")) {
    all = 1;
    return 0;
  }

  if ((ret=string_new(&v->s, c, &h, &s)) == -1)
    return -1;

  if (ret == 0)
    string_feed_slot(&v->s, h, &s);

  return 0;
}

int verbose_print(char *reg, FILE *f, char *s, ...)
{
  va_list ap;
  int ret;

  if (!inited) {
    fprintf(stderr, "verbose.c:verbose_print:%d: error : verbose not initialized\n", __LINE__);
    abort();
  }

  if (!all && string_search(&the_verbose->s, reg, 0, 0) == -1)
    return 0;

  va_start(ap, s);
  ret = vfprintf(f, s, ap);
  va_end(ap);

  return ret;
}

/* returns 1 if c is verbosable, 0 if not */
int verbose_on(char *c)
{
  if (!inited) {
    fprintf(stderr, "verbose.c:verbose_on:%d: error : verbose not initialized\n", __LINE__);
    abort();
  }

  if (all)
    return 1;

  return string_search(&the_verbose->s, c, 0, 0) != -1; 
}

#endif /* (__STDC_VERSION__ && __STDC_VERSION__ >= 199901L && DEBUG) || !(__STDC_VERSION__ && __STDC_VERSION__ >= 199901L) */
