#ifndef _JENKINS_HASH_H_
#define _JENKINS_HASH_H_

unsigned long jenkins_hash(unsigned char *, unsigned long, unsigned long);

/*
 This works on all machines.  hash2() is identical to hash() on
 little-endian machines, except that the length has to be measured
 in ub4s instead of bytes.  It is much faster than hash().  It
 requires
 -- that the key be an array of ub4's, and
 -- that all your machines have the same endianness, and
 -- that the length be the number of ub4's in the key
 */
unsigned long jenkins_hash2(unsigned long *, unsigned long, unsigned long);

/*
 This is identical to hash() on little-endian machines (like Intel
 x86s or VAXen).  It gives nondeterministic results on big-endian
 machines.  It is faster than hash(), but a little slower than
 hash2(), and it requires
 -- that all your machines be little-endian
 */
unsigned long jenkins_hash3(unsigned char *, unsigned long, unsigned long);

#endif /* _JENKINS_HASH_H_ */
